<?php

namespace Drupal\workflows_field_diagram\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\workflows\Entity\Workflow;

/**
 * Plugin implementation of the 'Workflows Field Diagram Formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "workflows_field_diagram_formatter",
 *   label = @Translation("Workflows Field Diagram"),
 *   field_types = {
 *     "workflows_field_item"
 *   }
 * )
 */
class WorkflowsFieldDiagramFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'flowchart-direction' => 'TD',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $elements['flowchart-direction'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mermaid Flowchart Direction (not available yet)'),
      '#description' => $this->t('<b>This Option is not available yet!</b><br>
                                Possible FlowChart Orientations will be:
                                <ul>
                                  <li>TB - top to bottom</li>
                                  <li>TD - top-down/ same as top to bottom</li>
                                  <li>BT - bottom to top</li>
                                  <li>RL - right to left</li>
                                  <li>LR - left to right</li>
                                </ul>'),
      '#default_value' => $this->getSetting('flowchart-direction'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary[] = $this->t('Mermaid Flowchart Direction: @flowchart-direction', ['@flowchart-direction' => $this->getSetting('flowchart-direction')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    // load workflow
    $workflow_id = $this->fieldDefinition->getSetting('workflow');
    $workflow = Workflow::load($workflow_id);
    $workflowType = $workflow->getTypePlugin();
    $states = $workflowType->getStates();

    foreach ($items as $delta => $item) {

      // mark active state
      $active_state_id = $item->value;

      // pass classes to states
      $classes = [
        'states' => [
          $active_state_id => ['active-state'],
        ],
        'transitions' => [],
      ];

      // disable states or transitions,
      $disabled = [
        'states' => [],
        'transitions' => [],
      ];

      $element[$delta] = [
        '#theme' => 'workflows_diagram',
        '#workflow' => $workflow,
        '#classes' => $classes,
        '#disabled' => $disabled,
        '#attached' => [
          'library' => [
            'workflows_field_diagram/workflows_field_diagram',
          ],
        ],
      ];
    }

    return $element;
  }

}
